import {IPCHandler} from "./IPCHandler";
import {FunctionTest, WiFiNetwork} from "./IPCConstants";
import OSHandler from "./OSHandler";
import WiFi from "./web/WiFi";

const wifiScan = require("node-wifi-scanner");


/**
 * Background Class for doing stuff on the host
 */
export default class SmartMonopoly {
    static run() {
        this.setupIPCEvents();
        OSHandler.enableAllWifis().then().catch(console.error);
    }

    static setupIPCEvents() {
        IPCHandler("FUNCTION_TEST", async (e, request, args) => {
            let data: FunctionTest = {
                hasSudo: await OSHandler.checkForSudo(),
                hasWPASupplicant: false,
                hasInternet: await OSHandler.checkForInternet(),
            };

            return {
                status: true,
                data: data
            }
        });

        IPCHandler("WIFI_CONNECT", async (e, request, args) => {
            let data = request.data as {ssid: string, psk: string}
            try {
                await OSHandler.addWifi(data.ssid, data.psk);
                let status = await OSHandler.connectToWifi(data.ssid);
                return {status: status};
            } catch(e)
            {
                return {status: false}
            }
        });

        IPCHandler("WIFI_SCAN", async (e, request, args) => {
            try {
                let networks = await OSHandler.scanWifis();
                return {status: true, data: networks};
            } catch(e)
            {
                return {status: false};
            }
        })
    }
}